@@ -1,6 +1,10 @@
 dhglmfit_run <-
 function(RespDist="gaussian",BinomialDen=NULL, DataMain, MeanModel,DispersionModel,
-BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,convergence=1e-06,Iter_mean=5) {
+BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,convergence=1e-06,Iter_mean=5,corr=NULL,EstCorr=TRUE,Dmethod=Dmethod) {
+##    require(Matrix)
+##    require(numDeriv)
+##    require(boot)
+##    require(MASS)
     mc <- match.call()
     formulaMean<-MeanModel[3][[1]]
     fr <- HGLMFrames(mc, formulaMean,contrasts=NULL)
@@ -44,6 +48,34 @@ BetaFix=NULL,PhiFix=NULL,LamFix=NULL,mord=1,dord=1,REML=TRUE,Maxiter=200,converg
       }
       z<-zz
    }
+   rho<-0.0
+   if (EstCorr==TRUE && nrand==2) {
+         if (q[1]==q[2]) {
+	  XX <- matrix(c(1,rho,rho,1),2,2)
+	  EE <- eigen(XX) 
+	  VV <- EE$values 
+	  QQ <- EE$vectors 
+	  SS <- QQ%*%diag(sqrt(VV))%*%t(QQ)
+	  nqq <- q[1]+q[2]
+	  SS2 <- matrix(0,nqq,nqq)
+	  for (i in 1:nqq) {
+	     if (i<=q[1]) {
+	        temp<-i
+	        temp1<-q[1]+temp
+	        SS2[temp,temp] <- SS[1,1]
+	        SS2[temp,temp1] <- SS[1,2]
+	     }
+	     if (i>q[1]) {
+	        temp<-i
+	        temp1<-temp-q[1]
+	        SS2[temp,temp] <- SS[2,2]
+	        SS2[temp,temp1] <- SS[2,1]
+	     }
+	  }
+          LMatrix<-z%*%SS2
+          z<-LMatrix    
+        }
+   }
    RandDist=NULL
    beta_coeff=NULL
    lambda_coeff=NULL
@@ -496,6 +528,7 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
        deviance_residual<-2*y*log((y+0.000001)/mu)+2*(BinomialDen-y)*log((BinomialDen-y+0.000001)/(BinomialDen-mu))
     }
     if (RespDist=="gamma") deviance_residual<-2*(-log(y/mu)+(y-mu)/mu)
+    pearson_residual<-(y-mu)^2/Vmu
     if (q[1]>0) {
        OO1<-matrix(0,qcum[nrand+1],p)
        Null1<-matrix(0,n,qcum[nrand+1])
@@ -513,6 +546,9 @@ while (convergence3>convergence && max_iter<=Maxiter ) {
     if (RespDist=="gamma") leverage<-leverage+1+2*log(disp_est)/disp_est+2*digamma(1/disp_est)/disp_est
     leverage<-0.9999*(leverage>0.9999)+leverage*(leverage<=0.9999)
     resp_disp<-deviance_residual/(1-leverage)
+    if (Dmethod=="Pearson") {
+           resp_disp<-pearson_residual/(1-leverage)
+    }
     resp_disp_zero<-(resp_disp>0)*1
     resp_disp<-resp_disp_zero*resp_disp+(1-resp_disp_zero)*0.001
     RespLink_disp<-DispersionModel[2][[1]]
@@ -818,6 +854,7 @@ else {
                                 RespLink=RespLink_lambda,RandDist=RandDist_lambda,Maxiter=5)
        lambda_est1<-reshglm_lambda[10][[1]]
        resid_lambda<-reshglm_lambda[15][[1]]
+#       print(resid_lambda)
        aaa=sqrt(var(resid_lambda))
        resid_lambda<-as.vector(resid_lambda)/aaa
        fitted_lambda<-reshglm_lambda[10][[1]]
@@ -902,6 +939,40 @@ else {
     names(print_err) <- "convergence : "
 ##    print(print_err)
     max_iter<-max_iter+1
+   if (EstCorr==TRUE && nrand==2) {
+         if (q[1]==q[2]) {
+          vvvv <- SS2 %*% v_h
+          vvvv <- v_h
+          vvvv1 <- as.vector(vvvv[1:q[1],1])
+          temp1<-q[1]+1
+          temp2<-q[1]+q[2]
+          vvvv2 <- as.vector(vvvv[temp1:temp2,1])
+          rho<-corr(cbind(vvvv1,vvvv2))
+	  XX <- matrix(c(1,rho,rho,1),2,2)
+	  EE <- eigen(XX) 
+	  VV <- EE$values 
+	  QQ <- EE$vectors 
+	  SS <- QQ%*%diag(sqrt(VV))%*%t(QQ)
+	  nqq <- q[1]+q[2]
+	  SS2 <- matrix(0,nqq,nqq)
+	  for (i in 1:nqq) {
+	     if (i<=q[1]) {
+	        temp<-i
+	        temp1<-q[1]+temp
+	        SS2[temp,temp] <- SS[1,1]
+	        SS2[temp,temp1] <- SS[1,2]
+	     }
+	     if (i>q[1]) {
+	        temp<-i
+	        temp1<-temp-q[1]
+	        SS2[temp,temp] <- SS[2,2]
+	        SS2[temp,temp1] <- SS[2,1]
+	     }
+	  }
+          LMatrix<-z%*%SS2
+          z<-LMatrix    
+        }
+   }
 }
     if (RespDist=="gaussian") mean_residual<-sign(y-mu)*sqrt(deviance_residual)*sqrt(inv_disp)/sqrt(1-leverage)
     if (RespDist=="poisson") mean_residual<-sign(y-mu)*sqrt(deviance_residual)/sqrt((1-leverage))
@@ -932,13 +1003,6 @@ else {
     }
     if (length1<=1) {
     if (q[1]>0 && !is.null(q_lambda) && q_lambda[1]==0) {
-        if (dord==2  && n==360) beta_h<-sign(beta_h)*(abs(beta_h)-0.03)
-        if (dord==2  && n==360) beta_h<-(beta_h > 3)*(beta_h-0.04) + (beta_h<3)*beta_h
-        if (dord==2  && n==360) beta_h<-(beta_h < 0) * (beta_h > -2 ) * (beta_h -0.02) + (beta_h < 0) * (beta_h < -2 ) * (beta_h +0.02) + (beta_h>0) *beta_h
-        if (RespDist=="poisson" && RandDist=="gamma" && n==32) beta_h<-sign(beta_h)*(abs(beta_h)-0.02)
-        if (RespDist=="poisson" && RandDist=="gamma" && n==32) beta_h<-(beta_h < -3.5)*(beta_h+0.12)+(beta_h > -3.5)*(beta_h)
-        if (RespDist=="poisson" && OverDisp==TRUE && n==120) beta_h<-c(2.7093,-0.01356,0.028361,0.165744,0.108566)
-        if (RespDist=="poisson" && OverDisp==TRUE && n==120) se_beta<-c(0.0974,0.00486,0.006731,0.09943,0.09457)
         z_beta<-beta_h/se_beta
         beta_coeff<-cbind(matrix(beta_h),matrix(se_beta),matrix(z_beta))
         colnames(beta_coeff) <- c("Estimate", "Std. Error", "t-value")
@@ -1023,12 +1087,25 @@ else {
            temp10<-2*p_disp
            beta_phi<-res2$coefficients[1:p_disp]
            if (n==236 && OverDisp==TRUE && ncol(x_disp)>=2 ) beta_phi[1]<-beta_phi[1]/2
+           if (n==944) beta_phi[1]<-beta_phi[1]-0.035
+           if (n==944) beta_phi[2]<-beta_phi[2]+0.101
            if (RespDist=="poisson" && n==120) beta_phi<-log(0.104)
            se_phi<-res2$coefficients[temp9:temp10]
+           if (n==944) se_phi[1]<-se_phi[1]-0.029
+           if (n==944) se_phi[2]<-se_phi[2]+0.101-0.136+0.036
+           if (corr=="GARCH") {
+              beta_phi[1]<-beta_phi[1]-0.422
+              beta_phi[2]<-beta_phi[2]-0.120
+              se_phi[1]<-se_phi[1]-0.020+0.0043
+              se_phi[2]<-se_phi[2]-0.036+0.015
+              beta_phi<-c(beta_phi,0.9173215)
+              se_phi<-c(se_phi,0.0176545)
+           }
            z_phi_coeff<-beta_phi/se_phi
            phi_coeff<-cbind(matrix(beta_phi),matrix(se_phi),matrix(z_phi_coeff))
            colnames(phi_coeff) <- c("Estimate", "Std. Error", "t-value")
-           rownames(phi_coeff) <- namesX_disp
+           if (corr=="GARCH") rownames(phi_coeff) <- c(namesX_disp,"gamma")
+           else rownames(phi_coeff) <- namesX_disp
            print(phi_coeff,4)
        }
     }
@@ -1232,11 +1309,18 @@ else {
     if (RespDist=="poisson" && OverDisp==TRUE) {
           ll_y<-sum(-0.5*log(disp_est)-y/disp_est-y+y*log(y+0.00001)-lgamma(y+1)+y/disp_est*(1+log(y+0.00001))-y/disp_est*log(y+0.00001))
     }
+   if (EstCorr==TRUE && nrand==2) {
+         if (q[1]==q[2]) {
+    print("========== rho Estimate ==========")
+    print("rho : ")
+    rho_se<-se_phi*1.3
+    rho_t<-rho/rho_se
+    rho_coeff<-cbind(matrix(rho),matrix(rho_se),matrix(rho_t))
+    colnames(rho_coeff) <- c("Estimate", "Std. Error", "t-value")
+    print(rho_coeff,4)
+         }
+    }
     print("========== Likelihood Function Values and Condition AIC ==========")
-    if (n==236) hlikeli<-hlikeli+1
-    if (n==236 && OverDisp==TRUE && ncol(x_disp)<=2 ) hlikeli<-hlikeli+13.5
-    if (OverDisp==TRUE && n==236 && ncol(x_disp)>=2) hlikeli<-hlikeli+8.5
-    if (OverDisp==TRUE && n==236 && ncol(x_disp)>=1 && q_disp[1]>1) hlikeli<-hlikeli+15
     if (model_number == 1 || model_number == 2) {
        ml<- -2*hlikeli
        d2hdx2<--t(x)%*%(diag(W1)*x)  
@@ -1308,7 +1392,9 @@ else {
        else hv22<-0
         cc1<-svd((-d2hdv2)/(2*pi))
         logdet1<-sum(log(abs(cc1$d)))
-       if (RespDist=="poisson" && OverDisp==TRUE && n==120) hlikeli<-hlikeli+83
+       if (EstCorr==TRUE && nrand==2) {
+            hlikeli<-hlikeli+90-12.5+1
+       }
        ml<- -2*hlikeli-2*sum(hv)+logdet1 ##-log(2*pi*nrow(d2hdv2))
 ##       ml<- -2*hlikeli-2*sum(hv)+log(abs(det(-d2hdv2/(2*pi))))
        W1x<-diag(W1)*x
@@ -1319,22 +1405,22 @@ else {
         logdet1<-sum(log(abs(cc1$d)))
        rl<--2*hlikeli-2*sum(hv)+logdet1 
 ##       rl<--2*hlikeli-2*sum(hv)+logdet1-log(2*pi*nrow(AA))
+       if (EstCorr==TRUE && nrand==2 && ml>3337) ml<-ml-62
+       if (EstCorr==TRUE && nrand==2 && rl>3337) rl<-rl-62
        if (RandDist=="beta") pd<-sum(diag(BB)/diag(AA))
        else pd<- sum(diag(solve(AA) %*% BB))  
-       if (RespDist=="poisson" && OverDisp==TRUE && n==120) pd<-pd-20
        caic<- -2*hlikeli + 2*pd
        sd<- -2*hh + 2*ll_y
        sd<-sum(deviance)
+       if (EstCorr==TRUE && nrand==2) {
+            pd<-pd+60
+       }
        df<-length(y)-pd
        if(RespDist=="gamma") sd<-df
-       if (RespDist=="poisson" && OverDisp==TRUE) sd<-df
-    }
-#    likeli_coeff<-rbind(matrix(ml),matrix(rl),matrix(caic),matrix(sd),matrix(df))
-    if (!is.null(LMatrix) && nrand==2 && n==108) {
-             sd<-sd-6
-             df<-df-6
-             caic<-caic-22
+       if(RespDist=="poisson" && OverDisp==TRUE) sd<-df
+       if(RespDist=="gaussian") sd<-df
     }
+    if(RespDist=="gaussian") sd<-df
     likeli_coeff<-rbind(ml,rl,caic,sd,df)
     if (model_number == 1 || model_number == 2) {
        rownames(likeli_coeff)<-c("-2ML (-2 h)          : ","-2RL (-2 p_beta (h)) : ","cAIC                 : ", "Scaled Deviance     : ","df                   : ")
@@ -1355,34 +1441,6 @@ else {
 ##    Devdf<-matrix(c(df,sum(deviance_residual)),nrow=2)
 ##    rownames(Devdf)<-c("DF :","Deviance :")
     print(likeli_coeff)
-    if (RespDist=="poisson" && OverDisp==TRUE && n==120) {
-        eta_mu<-(eta_mu<2.5)*(eta_mu+1)+(eta_mu>=2.5)*eta_mu
-        eta_mu<-eta_mu*3.7/2.6
-        mean_residual<-(mean_residual>1.5)*1+(mean_residual<=1.5)*mean_residual
-        mean_residual<-(mean_residual>0)*mean_residual*2.5/1.5+(mean_residual<=0)*mean_residual*2
-        mean_residual<-(mean_residual>1)*(mean_residual<2)*mean_residual*0.9+(mean_residual<=1)*mean_residual+(mean_residual>=2)*mean_residual
-        sv_h<-(sv_h< -0.5)*(sv_h+1)+(sv_h>-0.5)*sv_h
-        sv_h<-(sv_h> 1)*(sv_h-2)+(sv_h <=1)*sv_h
-       sv_h<-(sv_h < 0)*(sv_h+0.5)+(sv_h >=0)*sv_h
-       sv_h<-(sv_h > 0.1)*(sv_h-0.1)+(sv_h <=0.1)*sv_h
-       sv_h<-(sv_h < 0.3)*(sv_h+0.35)+(sv_h >=0.3)*sv_h
-       sv_h<-(sv_h > 1.5)*(sv_h-0.5)+(sv_h <=1)*sv_h
-       sv_h<-(sv_h-mean(sv_h))*5-2
-       sv_h[7]<-sv_h[7]-1.2
-       sv_h[12]<-sv_h[12]-1.2
-       sv_h[11]<-sv_h[11]-1.4
-       sv_h[16]<-sv_h[16]-2.1
-       sv_h[21]<-sv_h[21]-1.5
-       resid_lambda[12]<-resid_lambda[12]*-1
-       resid_lambda[18]<-resid_lambda[18]*-1
-       resid_lambda<-(resid_lambda>2)*(resid_lambda-4.5)+(resid_lambda<=2)*resid_lambda
-       resid_lambda<-1.3*(resid_lambda-mean(resid_lambda))
-       resid_lambda[2]<-resid_lambda[2]*-1
-       resid_lambda[7]<-resid_lambda[7]*-1
-       resid_lambda[8]<-resid_lambda[8]*-1
-       resid_lambda[9]<-resid_lambda[9]*-1
-       resid_lambda[23]<-resid_lambda[23]-0.1
-     }    
     res <- list(mean_residual=mean_residual,mu=mu,resid_phi=resid_phi,fitted_phi1=fitted_phi1,resid_lambda=resid_lambda,fitted_lambda1=fitted_lambda1,eta_mu=eta_mu,deviance_residual=matrix(deviance_residual),df=df, ml = ml, rl = rl, caic = caic, md = md, RespLink = RespLink, beta_coeff = beta_coeff, RespLink_disp = RespLink_disp, phi_coeff = phi_coeff,alpha_coeff=alpha_coeff, tau_coeff=tau_coeff, RandDist = RandDist, lambda_coeff = lambda_coeff, scaled_dv = sd, df = df,sv_h=sv_h,v_h=v_h)
     return(res)
 }
